@@ -14,7 +14,7 @@
 # | language governing permissions and limitations under the License.        |
 # +--------------------------------------------------------------------------+
 # | Authors: Swetha Patel                                                    |
-# | Version: 0.2.7                                                           |
+# | Version: 0.2.9                                                           |
 # +--------------------------------------------------------------------------+
 
 """
@@ -245,7 +245,7 @@ def _get_exception(inst):
     # These tuple are used to determine the type of exceptions that are
     # thrown by the database.  They store the SQLSTATE code and the
     # SQLSTATE class code(the 2 digit prefix of the SQLSTATE code)  
-    warning_error_tuple=('01')
+    warning_error_tuple=('01', )
     data_error_tuple=('02', '22', '10601', '10603', '10605', '10901', '10902', 
                                                                '38552', '54')
 
@@ -253,7 +253,7 @@ def _get_exception(inst):
                           '38503', '38553', '38H01', '38H02', '38H03', '38H04',
                                    '38H05', '38H06', '38H07', '38H09', '38H0A')
 
-    integrity_error_tuple = ('23')
+    integrity_error_tuple = ('23', )
 
     internal_error_tuple = ('24', '25', '26', '2D', '51', '57')
 
@@ -295,6 +295,7 @@ def _get_exception(inst):
                                                             "Fetch Failure: ",
                                                   "SQLNumResultCols failed: ",
                                                        "SQLRowCount failed: ",
+                                                   "SQLGetDiagField failed: ",
                                                  "Statement prepare Failed: ")
 
     operational_exceptions = (          "Connection Resource cannot be found", 
@@ -302,7 +303,7 @@ def _get_exception(inst):
                                                     "Describe Param Failed: ",
                                                  "Statement Execute Failed: ",
                                                       "Sending data failed: ", 
-                                     "Failed to Allocate Memory for XML Data", 
+                                     "Failed to Allocate Memory for XML Data",
                                      "Failed to Allocate Memory for LOB Data")
 
     # First check if the exception is from the database.  If it is 
@@ -387,9 +388,9 @@ def connect(dsn,user='',password='',host='',database='',conn_options=None):
             raise InterfaceError("connect expects the sixth argument"
                                  " (conn_options) to be of type dict")
         if not SQL_ATTR_AUTOCOMMIT in conn_options:
-            conn_options[SQL_ATTR_AUTOCOMMIT] = SQL_AUTOCOMMIT_ON
+            conn_options[SQL_ATTR_AUTOCOMMIT] = SQL_AUTOCOMMIT_OFF
     else:
-        conn_options = {SQL_ATTR_AUTOCOMMIT : SQL_AUTOCOMMIT_ON}
+        conn_options = {SQL_ATTR_AUTOCOMMIT : SQL_AUTOCOMMIT_OFF}
 
     # If the dsn does not contain port and protocal adding database
     # and hostname is no good.  Add these when required, that is,
@@ -493,21 +494,35 @@ class Connection(object):
     # Sets connection attribute values
     def set_option(self, attr_dict):
         """Input: connection attribute dictionary
-           Return: connection attribute value
+           Return: True on success or False on failure
         """
         return ibm_db.set_option(self.conn_handler, attr_dict, 1)
 
     # Retrieves connection attributes values
     def get_option(self, attr_key):
         """Input: connection attribute key
-           Return: connection attribute value
+           Return: current setting of the resource attribute requested
         """
         return ibm_db.get_option(self.conn_handler, attr_key, 1)
 
+    # Sets connection AUTOCOMMIT attribute
+    def set_autocommit(self, is_on):
+        """Input: connection attribute: true if AUTOCOMMIT ON, false otherwise (i.e. OFF)
+           Return: True on success or False on failure
+        """
+        try:
+          if is_on:
+            is_set = ibm_db.set_option(self.conn_handler, {SQL_ATTR_AUTOCOMMIT : SQL_AUTOCOMMIT_ON}, 1)
+          else:
+            is_set = ibm_db.set_option(self.conn_handler, {SQL_ATTR_AUTOCOMMIT : SQL_AUTOCOMMIT_OFF}, 1)
+        except Exception, inst:
+          raise _get_exception(inst)
+        return is_set
+
     # Sets connection attribute values
     def set_current_schema(self, schema_name):
         """Input: connection attribute dictionary
-           Return: connection attribute value
+           Return: True on success or False on failure
         """
         self.current_schema = schema_name
         try:
@@ -518,8 +533,7 @@ class Connection(object):
 
     # Retrieves connection attributes values
     def get_current_schema(self):
-        """Input: connection attribute key
-           Return: connection attribute value
+        """Return: current setting of the schema attribute
         """
         try:
           conn_schema = ibm_db.get_option(self.conn_handler, SQL_ATTR_CURRENT_SCHEMA, 1)
@@ -824,6 +838,7 @@ class Cursor(object):
         self.__description = None
         self.conn_handler = conn_handler
         self.stmt_handler = None
+        self._is_scrollable_cursor = False
 
     # This method closes the statemente associated with the cursor object.
     # It takes no argument.
@@ -879,6 +894,35 @@ class Cursor(object):
         except Exception, inst:
             raise _get_exception(inst)
 
+    # Helper for preparing an SQL statement.
+    def _set_cursor_helper(self):
+        self._result_set_produced = False
+        try:
+            ibm_db.set_option(self.stmt_handler, 
+                 {ibm_db.SQL_ATTR_CURSOR_TYPE: ibm_db.SQL_CURSOR_STATIC}, 0)
+        except Exception, inst:
+            raise _get_exception(inst)
+        try:
+            num_columns = ibm_db.num_fields(self.stmt_handler)
+        except Exception, inst:
+            raise _get_exception(inst)
+        if not num_columns:
+            return True 
+        self._is_scrollable_cursor = True
+        self._result_set_produced = True
+        if ibm_db.server_info(self.conn_handler).DBMS_NAME[0:3] != 'IDS':
+            for column_index in range(num_columns):
+                try:
+                    type = ibm_db.field_type(self.stmt_handler, column_index)
+                except Exception, inst:
+                    raise _get_exception(inst)
+                if type == "xml" or type == "clob" or type == "blob":
+                    self._is_scrollable_cursor = False
+                    print Warning("rowcount could not be updated because "
+                     "select includes xml, clob and/or blob type column(s)")
+                    return True
+        return True
+
     # Helper for executing an SQL statement.
     def _execute_helper(self, rows_counter, parameters=None):
         logger.debug('_execute_helper( '+str(rows_counter)+', '+repr(parameters)+' )')
@@ -922,43 +966,23 @@ class Cursor(object):
 
     # This method is used to set the rowcount after executing an SQL 
     # statement. 
-    def _set_rowcount(self, operation, parameters=None):
-        logger.debug('_set_rowcount( '+str(operation)+', '+str(parameters)+' )')
-        queryType = operation[0 : 6]
-        if queryType.lower() == 'select' and \
-           operation.lower().find('from') != -1:
-            temp_operation = 'select count(*) from (' + operation + ') as t'
+    def _set_rowcount(self):
+        logger.debug('_set_rowcount()')
+        self.__rowcount = 0
+        if not self._result_set_produced:
             try:
-                stmt_handler = ibm_db.prepare(self.conn_handler, temp_operation)
-                if parameters is not None:
-                    parameters = tuple(parameters)
-                    return_value = ibm_db.execute(stmt_handler, parameters)
-                    if not return_value:
-                        if ibm_db.conn_errormsg() is not None:
-                            raise Error(str(ibm_db.conn_errormsg()))
-                        if ibm_db.stmt_errormsg() is not None:
-                            raise Error(str(ibm_db.stmt_errormsg()))
-                else:
-                    return_value = ibm_db.execute(stmt_handler)
-                    if not return_value:
-                        if ibm_db.conn_errormsg() is not None:
-                            raise Error(str(ibm_db.conn_errormsg()))
-                        if ibm_db.stmt_errormsg() is not None:
-                            raise Error(str(ibm_db.stmt_errormsg()))
-                count_row_tuple = ibm_db.fetch_tuple(stmt_handler)
+                counter = ibm_db.num_rows(self.stmt_handler)
             except Exception, inst:
                 raise _get_exception(inst)
-
-            self.__rowcount = count_row_tuple[0]
-            self._result_set_produced = True
-            return True
-        else:
+            self.__rowcount = counter
+        elif self._is_scrollable_cursor:
             try:
-                return_value = ibm_db.num_rows(self.stmt_handler)
+                counter = ibm_db.get_num_result(self.stmt_handler)
             except Exception, inst:
                 raise _get_exception(inst)
-            self.__rowcount = return_value
-            return True
+            if counter >= 0:
+                self.__rowcount = counter
+        return True
 
     # Retrieves the last generated identity value from the DB2 catalog
     def _get_last_identity_val(self):
@@ -1007,8 +1031,9 @@ class Cursor(object):
         self.__description = None
         self._all_stmt_handlers = []
         self._prepare_helper(operation)
+        self._set_cursor_helper()
         self._execute_helper(None, parameters)
-        return self._set_rowcount(operation, parameters)
+        return self._set_rowcount()
 
 
     def executemany(self, operation, seq_parameters):
@@ -1178,4 +1203,4 @@ class Cursor(object):
 
                 except Exception, inst:
                     raise DataError("Data type format error: "+ str(inst))
-        return row
\ No newline at end of file
+        return row
